@@ -7,14 +7,15 @@ problems and solving them.
 
 import logging
 
-from optimization import OptimizationError,Problem
+from optimization import OptimizationError
 import get_data
 import powersystems
 import results
 import config
 from commonscripts import joindir,show_clock
-    
-def problem(datadir='.',
+import time as timer
+
+def solve_problem(datadir='.',
         shell=True,
         problemfile=False,
         visualization=True,
@@ -49,6 +50,7 @@ def problem(datadir='.',
     """
     
     _setup_logging(logging_level,logging_file)
+    start_time = timer.time()
     logging.debug('Minpower reading {} {}'.format(datadir, show_clock()))
     generators,loads,lines,times=get_data.parsedir(datadir)
     logging.debug('data read {}'.format(show_clock()))
@@ -70,23 +72,22 @@ def problem(datadir='.',
                                                        problemfile=problemfile,
                                                        )
         solution=results.make_multistage_solution(power_system,stage_times,datadir,stage_solutions)
-        logging.info('problem solved in {} ... finished at {}'.format(solution.solve_time,show_clock()))
     
     if shell: solution.show()
     if csv: solution.saveCSV()
     if visualization: solution.visualization()
     if solution_file: solution.save(solution_file)
+    logging.info('total time: {}s'.format(timer.time()-start_time))
     return solution
 
 def create_solve_problem(power_system,times,datadir,solver,problemfile=False,get_duals=True):
-    problem=create_problem(power_system,times)
+    create_problem(power_system,times)
     if problemfile: problemfile=joindir(datadir,'problem-formulation.lp')
-    problem.solve(solver,problem_filename=problemfile,get_duals=get_duals)
     
-    if problem.solved:
-        solution=results.make_solution(power_system,times,problem=problem,datadir=datadir)
-    else: 
-        raise OptimizationError('problem not solved')
+    power_system.solve(solver,problem_filename=problemfile,get_duals=get_duals)
+    solution=results.make_solution(power_system,times,datadir=datadir)
+    
+        
     return solution
 def create_problem(power_system,times):
     """
@@ -99,18 +100,16 @@ def create_problem(power_system,times):
     """
     
     
-    prob=Problem()
+    
     logging.debug('initialized problem {}'.format(show_clock()))
-    variables =power_system.create_variables(times)
+    power_system.create_variables(times)
     logging.debug('created variables {}'.format(show_clock()))
-    objective=power_system.create_objective(times)
+    power_system.create_objective(times)
     logging.debug('created objective {}'.format(show_clock()))
-    constraints=power_system.create_constraints(times)
+    power_system.create_constraints(times)
     logging.debug('created constraints {}'.format(show_clock()))
-    for v in variables.values(): prob.add_variable(v)
-    for c in constraints.values(): prob.add_constraint(c)
-    prob.add_objective(objective)
-    return prob
+    return
+
 
 def solve_multistage(power_system,times,datadir,
                               solver=config.optimization_solver,
@@ -158,28 +157,33 @@ def solve_multistage(power_system,times,datadir,
     def get_finalconditions(power_system,times):
         t_back=overlap_hours/times.intervalhrs
         next_stage_first_time = times[-1-int(t_back)]         
-        for bus in power_system.buses:
-            for gen in bus.generators:
-                gen.update_variables()
-                gen.finalstatus=gen.getstatus(t=next_stage_first_time,times=times)
+        for gen in power_system.generators():
+            gen.finalstatus=gen.getstatus(t=next_stage_first_time,times=times)
 
 
-    for t_stage in stage_times:
+    for stg,t_stage in enumerate(stage_times):
+        #print 'Stage starting at {}, {}'.format(t_stage[0].Start, show_clock(showclock))
         logging.info('Stage starting at {}, {}'.format(t_stage[0].Start, show_clock(showclock)))
         set_initialconditions(buses,t_stage.initialTime)
         
         try: stage_solution=create_solve_problem(power_system,t_stage,datadir,solver,problemfile,get_duals)
         except OptimizationError:
             #re-do stage, with load shedding allowed
-            logging.critical('Stage infeasible, re-running with load shedding.')
+            logging.critical('stage infeasible, re-running with load shedding.')
+            power_system.reset_model()
             power_system.set_load_shedding(True)
-            #save problem formulation for degbugging in case of infeasibility
-            stage_solution=create_solve_problem(power_system,t_stage,datadir,solver,problemfile=True,get_duals=get_duals)
+            try: 
+                stage_solution=create_solve_problem(power_system,t_stage,datadir,solver,problemfile=True,get_duals=get_duals)
+            except OptimizationError:
+                stage_solutions[-1].saveCSV(joindir(datadir,'last-stage-solved-commitment.csv'),save_final_status=True) 
+                raise OptimizationError('failed to solve, even with load shedding.')
             power_system.set_load_shedding(False)
 
         logging.debug('solved... get results... {}'.format(show_clock(showclock)))
         get_finalconditions(power_system,t_stage)
         stage_solutions.append(stage_solution)
+        
+        if stg<len(stage_times)-1: power_system.reset_model()
     return stage_solutions,stage_times
 
   
@@ -188,3 +192,72 @@ def _setup_logging(level,filename=False):
     if filename:
         logging.basicConfig( level=level, format='%(levelname)s: %(message)s',filename=filename)
     else: logging.basicConfig( level=level, format='%(levelname)s: %(message)s')
+
+
+def main():
+    '''
+    The command line interface for minpower. Described by:
+    minpower --help
+    '''
+    import argparse,os,traceback,sys
+
+    parser = argparse.ArgumentParser(description='Minpower command line interface')
+    parser.add_argument('directory', type=str, 
+                       help='the direcory of the problem you want to solve (or name of minpower demo case)')
+    parser.add_argument('--solver','-s',  type=str, default=config.optimization_solver,
+                       help='the solver name used to solve the problem (e.g. cplex, gurobi, glpk)')
+    parser.add_argument('--visualization','-v',action="store_true", default=False,
+                      help='save a visualization of the solution')
+    parser.add_argument('--breakpoints','-b',  type=int, default=config.default_num_breakpoints,
+                       help='number of breakpoints to use in piece-wise linearization of polynomial costs')
+    parser.add_argument('--commitment_hours','-c', type=int, default=config.default_hours_commitment,
+                       help='number hours per commitment in a rolling UC (exclusive of overlap)')
+    parser.add_argument('--overlap_hours','-o', type=int, default=config.default_hours_commitment_overlap,
+                       help='number hours to overlap commitments in a rolling UC')      
+    parser.add_argument('--problemfile','-p',action="store_true", default=False,
+                       help='flag to write the problem formulation to a problem-formulation.lp file -- useful for debugging')
+    parser.add_argument('--duals_off','-u',action="store_true", default=False,
+                      help='flag to skip getting the the duals, or prices, of the optimization problem')
+    parser.add_argument('--dispatch_decommit_allowed','-d', action="store_true", default=False,
+                        help='flag to allow de-commitment of units in an ED -- useful for getting initial conditions for UCs')
+    parser.add_argument('--logfile','-l',type=str,default=False,
+                       help='log file, default is to log to terminal')
+    parser.add_argument('--solution_file',type=str,default=False,
+                       help='save solution file to disk')
+    parser.add_argument('--profile',action="store_true",default=False,help='run cProfile and output to minpower.profile')
+    parser.add_argument('--error','-e',action="store_true",default=False,help='redirect error messages to the standard output (useful for debugging on remote machines)')
+                    
+    #figure out the command line arguments
+    args = parser.parse_args()
+
+    directory=args.directory
+
+    if not os.path.isdir(directory):
+        msg='There is no folder named "{}".'.format(directory)
+        raise OSError(msg)
+
+    inputs=dict(solver=args.solver,
+              num_breakpoints=args.breakpoints,
+              hours_commitment=args.commitment_hours,
+              hours_commitment_overlap=args.overlap_hours,
+              dispatch_decommit_allowed=args.dispatch_decommit_allowed,
+              visualization=args.visualization,
+              get_duals=not args.duals_off,
+              problemfile=args.problemfile,
+              logging_file=args.logfile,
+              solution_file=args.solution_file)
+    if args.profile:
+        print 'run profile'
+        import cProfile
+        prof = cProfile.Profile()
+        prof.runcall(solve.problem, directory, **inputs)
+        prof.dump_stats('minpower.profile')
+
+    else:
+        #solve the problem with those arguments
+        try: solve_problem(directory,**inputs)
+        except:
+            if args.error: 
+                print 'There was an error:'
+                traceback.print_exc(file=sys.stdout)
+            else: raise
\ No newline at end of file
