@@ -31,7 +31,10 @@ def make_loads_times(Pd=200,Pdt=None,**kwargs):
     
     return dict(loads=loads,times=times)
 
-def solve_problem(generators,loads=None,times=None, gen_init=None, lines=None, solver=config.optimization_solver,load_shedding_allowed=False,problem_filename=False):
+def solve_problem(generators,loads=None,times=None, gen_init=None, lines=None, 
+                  solver=config.optimization_solver,load_shedding_allowed=False,problem_filename=False,
+                  get_duals=False
+                  ):
     if lines is None: lines=[]
 
     if len(times)>0: 
@@ -43,12 +46,12 @@ def solve_problem(generators,loads=None,times=None, gen_init=None, lines=None, s
     
     
     power_system=powersystems.PowerSystem(generators,loads,lines,load_shedding_allowed=load_shedding_allowed)
-    problem=solve.create_problem(power_system,times)
-    problem.solve(solver=solver,problem_filename=problem_filename)
-    if problem.solved:
+    solve.create_problem(power_system,times)
+    power_system.solve(solver=solver,problem_filename=problem_filename,get_duals=get_duals)
+    if power_system.solved:
         power_system.update_variables()
     else:
         #logging.critical( [g.power[times.initialTime] for g in generators] )
-        problem.write('problem.lp')
+        power_system.write_model('problem.lp')
         raise optimization.OptimizationError('infeasible problem, wrote to problem.lp')
-    return power_system,times
+    return power_system,times
\ No newline at end of file
