@@ -22,7 +22,7 @@ def prices():
         make_mid_gen(Pmax=20),
         make_expensive_gen()
     ]
-    power_system,times=solve_problem(generators,**make_loads_times(Pdt=[80,110,130]))
+    power_system,times=solve_problem(generators,get_duals=True,**make_loads_times(Pdt=[80,110,130]))
     lmps = [power_system.buses[0].price(t) for t in times]
     
     assert lmps==[gen_costs['cheap'],gen_costs['mid'],gen_costs['expensive']]
@@ -55,7 +55,7 @@ def load_shedding():
     Pdt1=211
     generators=[make_cheap_gen(Pmax=Pmax)]
     Pdt=[110,Pdt1,110]
-    power_system,times=solve_problem(generators,load_shedding_allowed=True,**make_loads_times(Pdt=Pdt))
+    power_system,times=solve_problem(generators,get_duals=True,load_shedding_allowed=True,**make_loads_times(Pdt=Pdt))
     load=power_system.loads()[0]
     load_t1=load.power(times[1])
     load_t1_shed=load.shed(times[1])
